;; CK's .emacs File 

;; WARNING: the code below is not well organized, not well documented,
;; and relies on some packages that need to be installed. I'm not
;; tremendously proud of my .emacs but I do find it useful. The
;; lila-theme.el and class-fonts-theme.el theme files mentioned in the
;; firest few functions are provided in separate .el files.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fonts and font functions

(defun disable-all-themes ()
  "Disable all themes currently in use"
  (interactive)
  (mapc 'disable-theme custom-enabled-themes))

(defun refonts ()
  "Swap back to default colors"
  (interactive)
  (disable-all-themes)
  (load-theme 'lila t))

(defun class-fonts ()
  "Swap the color scheme to be suitable for projector presentations"
  (interactive)
  (disable-all-themes)
  (load-theme 'class-fonts t))

(defun class-fonts2 ()
  "Swap the color scheme to be suitable for projector presentations"
  (interactive)
  (disable-all-themes)
  (load-theme 'class-fonts2 t))

(refonts)

;; (add-hook 'after-init-hook 'refont)  ;; possible mechanisms to apply themes/fonts after initialization completes
;; (add-hook 'after-init-hook 'refont)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Load the elpa packages, required for some modes
(require 'package)
(add-to-list 'package-archives '("melpa" . "http://melpa.milkbox.net/packages/") t)
(package-initialize)

;; GLOBAL MODES 
(global-font-lock-mode 1)
(transient-mark-mode -1)
(show-paren-mode 1)
(tool-bar-mode 0)
(menu-bar-mode 0)
(blink-cursor-mode 0)
(scroll-bar-mode -1)
(global-auto-revert-mode 1) ;; Reload files that change on disk
(column-number-mode)
(setq-default truncate-lines t)
(server-start)
(fringe-mode '(0 . 0))
(global-hl-line-mode 1)
(ido-mode 1)
(setq-default indent-tabs-mode t) ; http://www.emacswiki.org/emacs/NoTabs
(setq compilation-scroll-output t)      ;scroll compilation to follow process

(undo-tree-mode -1)                     ;turn off undo-tree
;; (which-key-mode 1)                      ;don't turn on which-key
;; (which-func-mode) 
;; (recentf-mode 1)
;; (hs-minor-mode 1)

;; KEYBINDINGS
(global-set-key "\C-cc" (quote compile))
;(global-set-key "\M-;" (quote comment-dwim)) ;; now default
(global-set-key "\C-cu" 'uncomment-region)
(global-set-key "\C-cr" 'comment-region)
(global-set-key "\C-cf" (quote search-forward))
(global-set-key "\C-cg" (quote goto-line))
(global-set-key (kbd "M-g l") 'goto-line)
(global-set-key "\C-cp" (quote replace-string))
(global-set-key "\C-cq" (quote query-replace))
(global-set-key "\C-h\C-a" (quote apropos))
(global-set-key "\C-ct" (quote transient-mark-mode))
(global-set-key "\C-ca" (quote repeat-complex-command))
(global-set-key "\C-c\C-r" (quote revert-buffer))
(global-set-key "\M-o" (quote other-window))
(global-set-key "\C-cb" 'set-background-color)
(global-set-key "\C-x\C-b" (quote buffer-menu))
(global-set-key "\C-xk" (quote kill-current-buffer))
(global-set-key "\C-cl" 'auto-fill-mode)
(global-set-key "\C-cs" 'string-insert-rectangle)
(global-set-key "\M-j" (quote join-line))
(global-set-key "\M-g\M-g" 'font-lock-fontify-buffer)
(global-set-key "\C-c\C-t" 'toggle-truncate-lines)
(global-set-key "\C-c\M-l" 'org-store-link)
(global-set-key "\C-c\C-v" 'view-mode)
(global-set-key "\C-xg" 'magit-status)
(global-set-key "\C-xp" 'proced)
(define-key ctl-x-4-map "t" 'toggle-window-split)
(global-set-key "\C-ck" 'browse-kill-ring)
(global-set-key "\C-cv" 'find-file-at-point)
(global-set-key "\C-cn" 'copy-buffer-filename)
(global-set-key "\C-cj" 'shell-jump-to-current-directory)
(global-set-key "\C-cd" 'set-comment-column)
(global-set-key (kbd "C-+") 'increment-number-at-point)
(global-set-key "\343"  'butterfly)

;; QUICK FILES 
(set-register ?e '(file . "~/.emacs"))	;; Set the 'e' register to jump to .emacs by doing C-x r j e
(set-register ?b '(file . "~/.bashrc")) ;; Set the 'b' register to jump to .bashrc by doing C-x r j e
(set-register ?w '(file . "~/Dropbox/worklog"))
(set-register ?k '(file . "~/Dropbox/projects/keyboard/README.org"))
(set-register ?l '(file . "~/Dropbox/misc/longquotes"))
(set-register ?o '(file . "~/teaching/2041-F2018/ocaml-4.06-refman.txt"))
(set-register ?O '(file . "~/.octaverc"))
(set-register ?r '(file . "~/Dropbox/configs/.Rprofile"))
(set-register ?a '(file . "~/Dropbox/configs/awesome/rc.lua"))
(set-register ?c '(file . "~/Dropbox/accounts.gpg"))

;; enable some disabled commands
(put 'narrow-to-region 'disabled nil)
(put 'narrow-to-page 'disabled nil)
(put 'scroll-left 'disabled nil)

;; Variable Settings
(setq initial-scratch-message ";; Scratch that!\n")
(fset 'yes-or-no-p 'y-or-n-p) ;; make y/n suffice for yes/no q

;; Usually only use one space after periods
(set-variable 'sentence-end "[.?!][]\"')}]*\\($\\| $\\|	\\|  \\| \\)[ 	\n]*")

;; Certain names are text files, makefiles, etc.
(add-to-list 'auto-mode-alist '("README$" . text-mode))
(add-to-list 'auto-mode-alist '("Makefile\\.include\\|OCamlMakefile" . makefile-mode))
(add-to-list 'auto-mode-alist '("worklog\\|hacking\\|notation\\|NOTES\\|accounts\\.gpg" . org-mode))
(add-to-list 'auto-mode-alist '("\\.tbl" . org-mode))
(add-to-list 'auto-mode-alist '("\\.cgi$" . perl-mode))
(add-to-list 'auto-mode-alist '("\\.h$" . c++-mode))
(add-to-list 'auto-mode-alist '("\\.job$" . sh-mode))
(add-to-list 'auto-mode-alist '("\\.g4$" . antlr-mode))
(add-to-list 'auto-mode-alist '("\\.[sS]$" . asm-mode))
(add-to-list 'auto-mode-alist '("\\.ml$" . tuareg-mode))
(add-to-list 'auto-mode-alist '("\\.ino$" . c-mode))
(add-to-list 'auto-mode-alist '("\\.m$" . octave-mode))
;(add-to-list 'auto-mode-alist '("\\.mlt$" . tuareg-mode))

;; Location of local libraries: 
(add-to-list 'load-path "~/.emacs.d/elisp")
(add-to-list 'exec-path "~/bin")
(add-to-list 'exec-path "~/Dropbox/bin")

;; Use Google Chrome
(setq browse-url-browser-function 'browse-url-generic)
(setq browse-url-generic-program "chromium")

;; Info path
(require 'info)
;; (add-to-list Info-directory-list "some-dir")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; FUNCTIONS that don't fit anywhere in particular

(defun kill-current-buffer ()
  "Kill the current buffer."
  (interactive)
  (kill-buffer (current-buffer)))

(defun kill-no-confirm ()
  "Kill the current buffer without confirming"
  (interactive)
  (let ((kill-buffer-query-functions nil)) ;override normal queries which ask y/n for buffers with processes
    (kill-this-buffer)))

(require 'term)
(defun term-quit ()
  (interactive)
  (let ((kill-buffer-query-functions nil)) ;override normal queries which ask y/n for buffers with processes
    (term-kill-subjob)
    (kill-this-buffer)))
  
(defun top ()
  "Run top in a term buffer, 'q' kills the buffer"
  (interactive)
  (let ((topbuf (generate-new-buffer "*top*")))
    (other-window 1)
    (switch-to-buffer topbuf)
    (term-mode)
    (term-exec topbuf "top" "top" nil nil)
    (term-char-mode)
    (local-set-key "q" 'term-quit)))

(defun align-comments ()
  "Align all comments in the region to the current comment-column"
  (interactive)
  (save-excursion
    (save-restriction
      (narrow-to-region (mark) (point))
      (beginning-of-buffer)
      (while (search-forward comment-start nil 't)
        (comment-indent)))))

(defun dired-find-directory-at-point ()
  "In dired-mode, return the name of closest directory preceding
the point."
  (interactive)
  (if (equal major-mode 'dired-mode)
      (save-excursion
        (move-end-of-line 1)                  ; in case on first line
        (search-backward "  /"  nil 't)
        (forward-char 2)
        (let* ((beg (point))
               (end (progn (move-end-of-line 1)
                           (backward-char 1)
                           (point))))
          (buffer-substring beg end)))
    (error "Not dired-mode")))

(defun shell-jump-to-current-directory ()
  "Open *shell* and change to directory of the given
buffer. Start's shell if needed. Handles dired-mode specially to
look for the 'closest' subdirectory when multiple directories are
open."
  (interactive)
  (let* ((dir (if (equal major-mode 'dired-mode)          ; using dired?
                  (dired-find-directory-at-point)         ; dired mode: find closest directory
                  default-directory))                     ; other buffer: use default dir for buffer
         (dir (replace-regexp-in-string " " "\\\\ " dir)) ; escape spaces
	 (cmd (concat "cd " dir)))
    (when (one-window-p)
      (split-window-right))
    (if (get-buffer "*shell*")
      (progn
        (other-window 1)
        (switch-to-buffer "*shell*"))
      (progn
        (shell)
        ;; (accept-process-output)      ;doesn't seem to work as desired
        (sleep-for 0 10)))              ;10 millisec delay to let the shell get started before sending input (hack)
    (end-of-buffer)
    (insert cmd)
    (comint-send-input)))    

(defun random-float ()
  "Insert a random float between 0 and 1"
  (interactive)
  (insert (format "%.2f" (/ (float (random 10000)) 10000))))

(defun insert-date-heading ()
  "Insert the date at the point; useful in worklog for dated entries"
  (interactive)
  (insert (format-time-string "* %m/%d/%Y %A\n\n")))

(defun insert-date ()
  "Insert the date from shell's dat command"
  (interactive)
  (shell-command "date" (quote (4)) nil)
  (end-of-line)
  (delete-char 1)
  (insert " "))

(defun tabw (arg)
  "Set the tab width"
  (interactive "nSet tab width: ")
  (set-variable (quote tab-width) arg))

(defun insert-filename () 
  "Insert a filename interactively, start at current working directory"
  (interactive)
  (insert (read-file-name "Insert file:" default-directory)))

(defun copy-buffer-filename ()
  "Copy the current buffer file or directory name to the kill ring"
  (interactive)
  (let* ((bf (cond (buffer-file-name)
                   (dired-directory)
                   (default-directory)
                   '()))
         (bft (cond (bf (file-truename bf))
                    '())))
    (if bft
        (progn (kill-new bft)
               (message bft))
      (message "No file associated with buffer"))))

(defun show-matching-paren-line ()
  "Echo the matching line to a closing paren/brace."
  (let ((prev-char (char-before (point)))
        (end-line (line-number-at-pos)))
    (when (and prev-char (char-equal (char-syntax prev-char) ?\)))
      (save-mark-and-excursion
        (backward-sexp)
        (when (not (= end-line (line-number-at-pos)))
          (move-beginning-of-line nil)
          (let ((beg (point)))
            (move-end-of-line nil)
            (message "Matches: %s" (buffer-substring beg (point)))))))))

;; Enable off-screen display of matching parens
;; (advice-add 'show-paren-function :after #'show-matching-paren-line)
;(advice-remove 'show-paren-function #'show-matching-paren-line)

(defun sudo-edit (&optional arg)
  ;; http://emacsredux.com/blog/2013/04/21/edit-files-as-root/
  "Edit currently visited file as root.

With a prefix ARG prompt for a file to visit.
Will also prompt for a file to visit if current
buffer is not visiting a file."
  (interactive "P")
  (if (or arg (not buffer-file-name))
      (find-file (concat "/sudo:root@localhost:"
                         (ido-read-file-name "Find file(as root): ")))
    (find-alternate-file (concat "/sudo:root@localhost:" buffer-file-name))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MODE CUSTOMIZATIONS 

;; GOD MODE
(defun my-update-cursor ()
  (if (or god-local-mode)
      (set-cursor-color "red")
      (set-cursor-color "white")))
(add-hook 'god-mode-enabled-hook 'my-update-cursor)
(add-hook 'god-mode-disabled-hook 'my-update-cursor)
(global-set-key [C-tab] (quote god-local-mode))
; (global-set-key (kbd "<escape>") 'god-local-mode) ; don't switch to god-mode

;; IDO MODE 

;; ido mode preferred orders
(defun move-zips-to-end ()
  "Move zip files to the end of ido file lists"
  (let* ((is-zip?  (lambda (x) (string-match-p "\.zip" x)))
         (not-zips  (seq-remove is-zip? ido-temp-list))
         (the-zips (seq-filter is-zip? ido-temp-list))
         (re-order (append not-zips the-zips)))
    (setq ido-temp-list re-order)))
(add-hook 'ido-make-file-list-hook #'move-zips-to-end)

;; ido mode preferred orders
(defun re-order-ido-compare (x y)
  "Comparison function to re-order the listing of files to put
org files first, zips last, and otherwise be alphabetic."
  (let ((x-org (string-match-p "\.org" x))
        (y-org (string-match-p "\.org" y))
        (x-zip (string-match-p "\.zip" x))
        (y-zip (string-match-p "\.zip" y)))
    (cond ((and x-org (not y-org)) 't)
          ((and (not x-org) y-org) nil)
          ((and x-zip (not y-zip)) nil)
          ((and (not x-zip) y-zip) 't)
          (t (string< x y)))))

(defun re-order-ido ()
  "Sort the lists of files produced by ido according to the
preferences in re-order-ido-compare"
  (setq ido-temp-list
        (sort ido-temp-list #'re-order-ido-compare)))
(add-hook 'ido-make-file-list-hook #'re-order-ido)

;; Expand region(u
(use-package expand-region
  :ensure t
  :config (global-set-key (kbd "C-=") 'er/expand-region))
  
;; VIEW MODE
;; Don't like q and it's ilk messing up my windowing arrangement 
(add-hook 'view-mode-hook
 '(lambda ()
    (substitute-key-definition 'View-quit 'kill-this-buffer view-mode-map)
    (define-key view-mode-map "j" 'View-scroll-line-forward)
    (define-key view-mode-map "k" 'View-scroll-line-backward)
))

;; BROWSE THE KILL RING
(require 'browse-kill-ring)

;; Easy encryption
(require 'epa-file)
(defadvice epg--start (around advice-epg-disable-agent disable)
  "Make epg--start not able to find a gpg-agent, prompt in minibuffer"
  (let ((agent (getenv "GPG_AGENT_INFO")))
    (setenv "GPG_AGENT_INFO" nil)
    ad-do-it
    (setenv "GPG_AGENT_INFO" agent)))
(ad-enable-advice 'epg--start 'around 'advice-epg-disable-agent)
(ad-activate 'epg--start)

;; TEXT HOOK
(add-hook 'text-mode-hook
 '(lambda ()
    (flyspell-mode 1)
    (local-set-key "\C-c\C-d" 'insert-date)
    (local-set-key "\C-cz" 'format-float)
    (auto-fill-mode 1)
))

;; OUTLINE HOOK
(add-hook 'outline-mode-hook
 '(lambda ()
    (local-set-key "\M-n" 'outline-next-visible-heading)
    (local-set-key "\M-p" 'outline-previous-visible-heading)
    (local-set-key (quote [f12]) (quote ck-outline-toggle))    
    (local-set-key "\M-(" (quote hide-body))
    (local-set-key "\M-)" (quote show-all))
))

;; LILYPOND for music editing
(load-library "lilypond-init")

;; HELM
;; (helm-mode 1) (helm-mode 0)		;keep using?

;; ;; Swap these as tab is faster for completion
;; (define-key helm-map (kbd "C-i") 'helm-execute-persistent-action)
;; (define-key helm-map (kbd "C-z") 'helm-select-action)

;; Buffer names based on filename
(load-library "uniquify")

;; Whitespace mode
(load-library "whitespace")

;; AUCTEX

; own versions of next/prev heading as outline version doesn't allow
; spaces before outline headers in regexp which means \problem doesn't work
(defun next-outline-heading ()
  (interactive)
  (move-end-of-line 1)
  (search-forward-regexp outline-regexp)
  (search-backward-regexp outline-regexp))
  
(defun previous-outline-heading ()
  (interactive)
  (move-beginning-of-line 1)
  (search-backward-regexp outline-regexp))
  
(add-hook 'latex-mode-hook
 '(lambda ()
    ;; (local-set-key "\C-c/" 'LaTeX-close-environment)
    (local-set-key "\C-c/" 'latex-close-block)
    (outline-minor-mode 1)
    (local-set-key (quote [C-tab]) (quote outline-toggle-children))
    ;; (local-set-key "\M-n" 'outline-next-visible-heading)
    ;; (local-set-key "\M-p" 'outline-previous-visible-heading)
    (local-set-key "\M-n" 'next-outline-heading) 
    (local-set-key "\M-p" 'previous-outline-heading)
    (local-set-key (quote [f12]) (quote ck-outline-toggle))    
    (local-set-key "\M-(" (quote hide-body))
    (local-set-key "\M-)" (quote show-all))
    (outline-minor-mode 1)
    (setq outline-regexp (concat outline-regexp "\\|\\\\problem\\|\\\\pagebreak")) ;enable scanning through exam problems
    (setq TeX-outline-extra '(("\\\\begin\{frame\}" 4)))
))

;; LUA
(use-package lua-mode) ;; Appears broken

; SCHEME
; (require 'quack)    

;; OCAML and TUAREG MODE
(defvar ocaml-sources-cache nil)
(defun ocaml-sources-init ()
  (split-string (shell-command-to-string "grep -sh '^let [^_(]' *.ml |gawk '{print $2}'")))

(defvar ac-ocaml-sources
  '((init . ocaml-sources)
    (candidates . ocaml-sources-init)))

;; inexplicably not interactive in tuareg.el
(defun tuareg-fill-comment ()
  (interactive)
  (save-mark-and-excursion
   (tuareg--fill-comment)))

;; TUAREG MODE - improved ocaml editing
(add-hook 'tuareg-mode-hook
 '(lambda () 
    (local-set-key "\M-;" 'comment-dwim)
    (local-set-key "\M-q" 'tuareg-fill-comment)
    ;; (local-set-key "\M-q" 'fill-paragraph)
    ;; (local-set-key "\M-n" 'tuareg-next-phrase)
    ;; (local-set-key "\M-p" 'tuareg-previous-phrase)
))

;; DIRED: Associate programs for external running
(require 'run-assoc)
(setq associated-program-alist
      '(("evince" "\\.pdf$") ;("mupdf" "\\.pdf$")
	("evince" "\\.ps\\|\\.eps$")
	("inkscape" "\\.svg$")
	("openscad" "\\.scad$")
	("gimp" "\\.xcf$")
	("yed" "\\.graphml$")
	("gnumeric" "\\.xls")
	;; ("libreoffice" "\\.xls")
	("eog" "\\.\\(png\\|jpg\\|jpeg\\|gif\\|tif\\|tiff\\|webp\\)$")
	("totem" "\\.mp3$")
	("mplayer" "\\.ogg$")
	("mplayer" "\\.oga$")
	("totem" "\\.wmv$")
	("totem" "\\.wav$")
	;; ("rasmol" "\\.pdb$")
	("evince" "\\.ps$")
	("evince" "\\.ps.gz$")
	("libreoffice" "\\.docx?\\|\\.od[ts]$\\|\\.ppt\\|\\.wps")
	("gnumeric" "\\.gnumeric")
	("libreoffice" "\\.rtf")
	("totem" "\\.\\(mpg\\|mpeg\\)$")
        ("totem" "\\.\\(mov\\|\\mp4\\|\\flv\\|\\webm\\|mkv\\)$")
	((lambda (file)
	   (browse-url (concat "file:///" (expand-file-name file)))) "\\.html?$")))

;; DIRED HOOK
(require 'dired-sort)			;For sorting by size
(require 'dired-x)			;Omit dot files

(defun dired-kill-subdir ()
  "Kill the subdirectory that the point is currently in"
  (interactive)
  (dired-prev-subdir)
  (dired-do-kill-lines '(4)))


(add-hook 'dired-mode-hook
  (lambda ()
     (setq dired-omit-files "^\\.?#\\|^\\.$\\|^\\.\\.$\\|^\\.[a-zA-Z]")
     (dired-omit-mode 1)                   ;initially hide uninteresting files
     (local-set-key "h" 'dired-omit-mode)  ;toggle hiding 'uninteresting' files
     ;; (local-set-key "j" 'dired-next-line)
     ;; (local-set-key "k" 'dired-previous-line)
     (local-set-key "\M-o" 'other-window)
     (local-set-key "q" 'kill-this-buffer)
     (local-set-key "c" 'dired-do-copy-with-dwim)
     (local-set-key "r" 'dired-do-rename-with-dwim)
     (local-set-key "K" 'dired-kill-subdir)
     (local-set-key "\M-n" 'dired-next-subdir)
     (local-set-key "\M-p" 'dired-prev-subdir)
     (local-set-key "z" 'dired-zip-file)
     (local-set-key "\C-m" 'dired-run-associated-program)))

(defun dired-do-copy-with-dwim ()
  "Do a copy with a temporarily set dired-dwim-target which will
attempts to use adjacent dired buffers as the target of the
copy."
  (interactive)
  (let ((dired-dwim-target 't))
    (dired-do-copy)))

(defun dired-do-rename-with-dwim ()
  "Do a rename with a temporarily set dired-dwim-target which
will attempts to use adjacent dired buffers as the target of the
rename."
  (interactive)
  (let ((dired-dwim-target 't))
    (dired-do-rename)))

(defun dired-zip-file ()
  "Create an archive containing the marked files."
  (interactive)
  (let* ((file (file-name-nondirectory (dired-get-filename)))
         (zipfile (concat file ".zip"))
         (cmd (concat "zip " zipfile " " file " -rq ")))
    (shell-command cmd)
    (revert-buffer)))
    

;; ESS for R-mode
;; (load "~/packages/ess-5.3.8/lisp/ess-site")
;(add-to-list 'load-path  "~/.elisp/ess-5.14/lisp")
;(require 'ess-site)

;; PYTHON MODE
(require 'python)
(add-hook 'python-mode-hook
  '(lambda ()
     (local-set-key "
" 'newline-and-indent)))

;; JAVA
(use-package javadoc-lookup)

(add-hook 'java-mode-hook
  '(lambda ()
     (subword-mode)                     ;handle camel case
     (local-set-key "\C-hj" (quote javadoc-lookup))
     (local-set-key "\C-c\C-j" (quote imenu))
     (local-set-key [C-S-iso-lefttab] (quote hs-hide-level))
     (local-set-key [backtab] (quote hs-toggle-hiding))
     (hs-minor-mode 1)))

;; (javadoc-set-predefined-urls '("/usr/share/doc/java8-openjdk/api"
;;                                "/usr/share/doc/openjdk-6-jre-headless/api"
;;                                "/usr/share/doc/openjdk7-doc/api/"))))

;; Folding regex for java code
;; http://www.emacswiki.org/emacs/OutlineMinorMode#toc9
;; Seems to require some tweaking to work
; (setq outline-regexp "\\(?:\\([ \t]*.*\\(class\\|interface\\)[ \t]+[a-zA-Z0-9_]+[ \t\n]*\\({\\|extends\\|implements\\)\\)\\|[ \t]*\\(public\\|private\\|static\\|final\\|native\\|synchronized\\|transient\\|volatile\\|strictfp\\| \\|\t\\)*[ \t]+\\(\\([a-zA-Z0-9_]\\|\\( *\t*< *\t*\\)\\|\\( *\t*> *\t*\\)\\|\\( *\t*, *\t*\\)\\|\\( *\t*\\[ *\t*\\)\\|\\(]\\)\\)+\\)[ \t]+[a-zA-Z0-9_]+[ \t]*(\\(.*\\))[ \t]*\\(throws[ \t]+\\([a-zA-Z0-9_, \t\n]*\\)\\)?[ \t\n]*{\\)" )

;; C MODE and derivatives
(add-hook 'c-mode-common-hook
 (lambda () 
   (setq comment-start "// ")
   (setq comment-end "")
   (c-set-offset 'cpp-macro 0 nil) ;; indent #pragmas
   (setq c-basic-offset 2)))

;; ASM MODE and derivatives
(setq asm-comment-char ?#) ;; done outside of hook to avoid reload
(add-hook 'asm-mode-hook
 (lambda () 
   (setq comment-start "# ")
   (setq comment-end "")))

;; AWK MODE
(add-hook 'awk-mode-hook
   (lambda ()
     (set-variable 'c-basic-offset 2 nil)
     (set-variable 'comment-start "#")
     (set-variable 'comment-end   "")
     ;; (set-variable 'comment-start "/* ")
     ;; (set-variable 'comment-end   " */")
     ))

;; GNUPLOT MODE 
;; (autoload 'gnuplot-mode "gnuplot" "gnuplot major mode" t)
;; (autoload 'gnuplot-make-buffer "gnuplot" "open a buffer in gnuplot mode" t)
;; (add-to-list 'auto-mode-alist '("\\.gp$" . gnuplot-mode))

;; F90 RECOGNITION
(add-to-list 'auto-mode-alist  '("\\.f$" . f90-mode))

;; MATLAB MODE
(add-to-list 'load-path "~/.emacs.d/elisp/matlab-emacs")
(require 'matlab-load)
(autoload 'matlab-mode "matlab" "Enter Matlab mode." t)
(add-to-list 'auto-mode-alist '("\\.[mM]$" . matlab-mode))
(autoload 'matlab-shell "matlab" "Interactive Matlab mode." t)

(defun swap-mat ()
  "Switch buffer name of matlab shell buffer"
  (interactive)
  (let ((nm (buffer-name)))
    (if (string-equal nm "*MATLAB*")
	(cond
	 ((not (get-buffer "mat1")) (rename-buffer "mat1"))
	 ((not (get-buffer "mat2")) (rename-buffer "mat2"))
	 ((not (get-buffer "mat3")) (rename-buffer "mat3")))
      (rename-buffer "*MATLAB*"))))

(defun matlab-eval-line ()
  "Evaluate the current line in the *MATLAB* buffer"
  (interactive)
  (save-excursion 
    (matlab-shell-run-region-or-line))
  (next-line))
(add-hook 
 'matlab-shell-mode-hook
 '(lambda ()
    (local-set-key "\C-cm" 'swap-mat)))
  
(add-hook 'matlab-mode-hook
  '(lambda ()
     (local-set-key "\C-c\C-n" 'matlab-eval-line)))

;; OCTAVE MODE
(add-hook 'octave-mode-hook
  '(lambda ()
     (set-variable 'comment-start "%" nil)
     (local-set-key "\C-c\C-n" 'octave-send-line)))

;; CLOJURE
(require 'clojure-mode)

(add-hook 'clojure-mode-hook #'cider-mode)

;; ADD COMPILE ERROR MESSAGE support for 'lein compile'.  This does
;; not work very well as the file names reported are always those
;; under the "src/" directory so the compilation must be run in that directory to 
(require 'compile)
(add-to-list 'compilation-error-regexp-alist-alist
             '(clojure-lein-compile "^Exception.*compiling:(\\(.*\\):\\(.*\\):\\(.*\\))" 1 2 3)) 
                                    ;                          file     line     col#
(add-to-list 'compilation-error-regexp-alist 'clojure-lein-compile)
;; see docs on variables for info on format of regexp: give match
;; group number for file, line, col.  Used
;; (progn
;;   (re-search-forward "^Exception.*compiling:(\\(.*\\):\\(.*\\):\\(.*\\))")
;;   (list (match-string 0) (match-string 1) (match-string 2) (match-string 3)))
;;  for testing


;; MAGIT for git interaction
(require 'magit)
(add-hook 'magit-mode-hook
  '(lambda () 
     (local-set-key "q" 'bury-buffer)))

;; EMACS PAGER: see the emacs-pager command line tool
(require 'ansi-color)
(defun page-file (filename)
  "Command to run to open a buffer for 'paging', just display the text for reading.

Most often used in an in-emacs shell when more/less would be used instead. "
  (interactive)
  (find-file filename)
  (ansi-color-apply-on-region (point-min) (point-max)))

;; ;; Introduces bugs which cause finished shell to be unable to restart
;; ;; errors: progn: Wrong type argument: processp, nil

;; SHELL MODE
(set-face-attribute 'comint-highlight-prompt nil :inherit nil)           ; allow color prompt from PS1 variable

;; PROCED MODE
;; Some Custom functions for proced
(defun pow2 (n)
  (let ((x 1))
    (while (> n 0)
      (setq x (* 2 x))
      (setq n (- n 1)))
    x))

(defun print-mem (kb)
  (cond ((< kb (pow2 10)) (format "%dK" kb))
	((< kb (pow2 20)) (format "%.1fM" (/ kb (float (pow2 10)))))
	((< kb (pow2 30)) (format "%.1fG" (/ kb (float (pow2 20)))))))

(setq proced-auto-update-interval 1)

(add-hook 'proced-mode-hook
  '(lambda ()
     (message "Running hook")
     (local-set-key "a" 'proced-toggle-auto-update)))


;; CSV EDITING

;; Appenda column of commas to the last column of the buffer"
(fset 'csv/append-column
   [?\C-x ?h ?\M-x ?r ?e ?p ?l ?a ?c ?e ?- ?r ?e ?g ?e ?x ?p return ?$ return ?, return ?\M-< ?\C-e])

(defun csv/align-buffer ()
  "Align entire buffer to commas"
  (interactive)
  (save-mark-and-excursion
   (replace-regexp " *, *" "," nil (point-min) (point-max))
   (align-regexp (point-min) (point-max) "\\(, *\\)" -1 0 t)))
  ;; (align-regexp (point-min) (point-max) "\\( *\\)\\(,\\)\\( *\\)" 3 0 t))
  ;; (align-regexp (point-min) (point-max) "\\s-*\\(,\\)\\s-*" 0 0 t))

(defun cvs/doto-single-column (funarg)
  "Apply the given funarg to the column in which the point
resides. Searches for the commas to operate on complete
spreadsheet columns."
  (push-mark)
  (save-mark-and-excursion
   (let ((col-end   (progn (search-forward-regexp  "$\\|,") (backward-char)  (current-column)))
         (col-begin (progn (search-backward-regexp "^\\|,") (current-column))))
     (message "begin %d end %d" col-begin col-end)
     (beginning-of-buffer)
     ;; (while (< (point) (point-max))
     (dotimes (i (- (line-number-at-pos (point-max)) 1))
       (goto-line (+ i 1))
       (let ((reg-begin (progn (move-to-column col-begin) (point)))
             (reg-end   (progn (move-to-column col-end)   (point))))
         (funcall funarg reg-begin reg-end)
       ))
     ))
  (csv/next-field))


(defun csv/hide-column ()
  "Hide the column of text where the point currently resides"
  (interactive)
  (cvs/doto-single-column 'facemenu-set-invisible))

(defun csv/show-columns ()
  " the column of text where the point currently resides"
  (interactive)
  (push-mark)
  (save-mark-and-excursion
   (let ((col-end   (progn (search-forward-regexp  "$\\|,") (backward-char)  (current-column)))
         (col-begin (progn (search-backward-regexp "^\\|,") (current-column))))
     (message "begin %d end %d" col-begin col-end)
     (beginning-of-buffer)
     ;; (while (< (point) (point-max))
     (dotimes (i (- (line-number-at-pos (point-max)) 1))
       (goto-line (+ i 1))
       (let ((invis-begin (progn (move-to-column col-begin) (point)))
             (invis-end   (progn (move-to-column col-end)   (point))))
         (facemenu-set-invisible invis-begin invis-end)
         ;; (message "making invisible line %d (%d to %d)" (line-number-at-pos) invis-begin invis-end)
         ;; (goto-char invis-begin) (insert ">")
         ;; (goto-char invis-end)   (insert "<")
                                        ; (next-line)
       ))
     )))

(defun csv/show-all-columns ()
  "Show all hidden columns"
  (interactive)
  (remove-text-properties (point-min) (point-max) '(invisible nil))) ; (facemenu-remove-all (point-min) (point-max) ))

(defun csv/next-field ()
  (interactive)
  (forward-char)
  (search-forward-regexp ",\\|^"))

(defun csv/previous-field ()
  (interactive)
  (backward-char)
  (search-backward-regexp ",\\|^")
  (when (= (char-after) ?,)
    (forward-char)))

(defvar csv-mode-map (make-sparse-keymap) "Keymap for csv-mode")
(define-key csv-mode-map (kbd "C-c C-c")   'csv/align-buffer)
(define-key csv-mode-map (kbd "C-c a")     'csv/append-column)
(define-key csv-mode-map (kbd "C-c h")     'csv/hide-column)
(define-key csv-mode-map (kbd "C-c s")     'csv/show-all-columns)
(define-key csv-mode-map (kbd "<tab>")     'csv/next-field)
(define-key csv-mode-map (kbd "<backtab>") 'csv/previous-field)

(define-minor-mode csv-mode
  "Toggle CSV editing mode."
  ;; The initial value.
  :init-value nil
  ;; The indicator for the mode line.
  :lighter " CSV"
  ;; The minor mode bindings.
  :keymap csv-mode-map
  ;; '(((kbd "C-c C-c") . csv/align-buffer)
  ;;   ([C-c a]   . csv/append-column)
  ;;   ([C-c h]   . csv/hide-column)
  ;;   ([C-c s]   . csv/show-all-columns)
  ;;   ((kbd "<tab>")     . csv/next-field)
  ;;   ((kbd "<backtab>") . csv/previous-field))
  :group 'csv)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ORG MODE
;; (require 'org-install)
;; (require 'ox-odt)                       ; odt export
;; (eval-after-load "org"                  ; Load additonal exports for markdown
;;   '(require 'ox-md nil t))


(set-variable (quote org-src-preserve-indentation) t nil) ; Makefiles can be incldued
(set-variable 'org-html-link-org-files-as-html nil)       ;disable annoying .org to .html link translation

;; controls which source blocks can execute to produce :results
(org-babel-do-load-languages
 'org-babel-load-languages
 '((emacs-lisp . nil)
   (shell . t)
   (python . t)))                        ;enable shell scripts to execute

;; ;; Don't want \alert in beamer
;; (defun my-beamer-bold (contents backend info)
;;   (when (eq backend 'beamer)
;;     (replace-regexp-in-string "\\`\\\\[A-Za-z0-9]+" "\\\\textbf" contents)))
;; (add-to-list 'org-export-filter-bold-functions 'my-beamer-bold)

(defun increment-number-at-point ()
  "Increment a number at the point"
  (interactive)
  ;; (skip-chars-backward "0-9")  ;; requires number before point
  (er/expand-region 1) ;; selects number before or after point
  (or (looking-at "[0-9]+")
      (error "No number at point"))
  (replace-match (number-to-string (1+ (string-to-number (match-string 0))))))

(defun org-export-latex-and-compile ()
  "Export current buffer as latex then compile in a separate
process; prevents blocking of emacs process"
  (interactive)
  (let* ((base (file-name-base))
	 (file (format "%s.tex" (file-name-sans-extension base)))
         (cmd (format "sed -i 's/\\\\usepackage{grffile}/%%\\\\usepackage{grffile} %%buggy package/g' %s" file))
	 (cmd (format "%s && xelatex -shell-escape -interaction nonstopmode %s" cmd file))
	 (cmd (format "%s && rm -f %s.{log,snm,toc,nav,aux,vrb,out} %s.*.vrb" cmd base base))
         (org-version-major (string-to-number (substring org-version 0 1 )))
	 ;; (cmd (format "pdflatex -shell-escape -interaction nonstopmode %s" file)) ;; old pdflatex with no unicode
	 ;(cmd (format "%s && ps -A |grep mupdf && killall -SIGHUP mupdf" cmd))
         )
    (when (file-exists-p file) 
      (delete-file file))
    (pcase org-version-major
      (9 (org-beamer-export-to-latex)) ;; for org 9
      (8 (org-beamer-export-to-latex)) ;; for org 8
      (7 (progn			       ;; for org 7
	   (org-export-as-latex nil) 
	   (kill-buffer file)))
      (_ (error "Unknown org version %d" org-version-major)))    
    (compile cmd)))

    ;; (message "Cleaning up auxilliary files")
    ;; (dolist (ext org-export-pdf-logfiles)
    ;;   (setq file (concat base "." ext))
    ;;   (message file)
    ;;   (and (file-exists-p file) (delete-file file)))
    ;; (message "Done")))

;; defines org export variables
(require 'ox)                           

;; In beamer export allow for bold via *word* rather than default red text
(defun my-beamer-bold (contents backend info)
  (when (eq backend 'beamer)
    (replace-regexp-in-string "\\`\\\\[A-Za-z0-9]+" "\\\\textbf" contents)))
(add-to-list 'org-export-filter-bold-functions 'my-beamer-bold)

;; In beamer export allow for red text via _word_
(defun my-beamer-underline (contents backend info)
  (when (eq backend 'beamer)
    (replace-regexp-in-string "\\`\\\\[A-Za-z0-9]+" "\\\\alert" contents)))
(add-to-list 'org-export-filter-underline-functions 'my-beamer-underline)

(defun org-example (latex)
  "Insert a code example in org-mode"
  (interactive "P")
  (message "%s" latex)
  (let ((type (read-string "Example type: ")))
    (if (use-region-p)
        (progn
          (insert (format "#+END_SRC\n"))
          (exchange-point-and-mark)
          (insert (format "#+BEGIN_SRC %s\n" type))
          (exchange-point-and-mark))
      (progn 
        (insert (format "#+BEGIN_SRC %s\n\n#+END_SRC\n" type))
        (previous-line 2)))))
    ;; (if latex
    ;;   (progn 
    ;;       (insert (format "#+LATEX:{\n#+BEGIN_SRC %s\n\n#+END_SRC\n#+LATEX:}\n" type))
    ;;       (previous-line 3))

(defun org-quote ()
  "Insert a quote in org-mode"
  (interactive)
  (insert (format "#+BEGIN_QUOTE\n\n#+END_QUOTE\n"))
    (previous-line 2))

(defun org-image ()
  "Insert centered, page-wide image in org-mode"
  (interactive)
  (insert (format "#+BEGIN_CENTER\n# #+ATTR_HTML: :width 100%% :style max-width:100%%;\n#+ATTR_LATEX: :width \\textwidth\n\n#+END_CENTER\n"))
    (previous-line 2))

(defun org-latex-tag ()
  "Insert a latex tag"
  (interactive)
  (insert (format "#+LATEX:")))

(defun org-html-tag ()
  "Insert a html tag"
  (interactive)
  (insert (format "#+HTML:")))

(defun org-two-column ()
  "Insert centered, page-wide image in org-mode"
  (interactive)
  (insert "#+LATEX:\\vspace{-.2in}
** Cols								  :B_columns:
   :PROPERTIES:
   :BEAMER_env: columns
   :BEAMER_OPT: 
   :END:

*** 
   :PROPERTIES:
   :BEAMER_col: .5
   :BEAMER_env: block
   :END:

*** 
   :PROPERTIES:
   :BEAMER_col: .5
   :BEAMER_env: block
   :END:
")
  (outline-previous-visible-heading 1)
  (org-cycle) (org-cycle)
  (outline-previous-visible-heading 1)
  (org-cycle) (org-cycle)
)

(defun org-html-image-attribute ()
  "Insert a html tag"
  (interactive)
  (insert "#+ATTR_HTML: :style max-width: 100%; width: 100%;"))

(defun org-latex-fontsize ()
  "Insert a \fontsize{}{} latex command"
  (interactive)
  (insert "#+LATEX:\\fontsize{}{}")
  (backward-char 3))

;; Can likely remove this
(defun org-set-html-pre-post ()
  "Set the two variables so HTML preamble and postamble are
correct. Overrides a bug in which #+BIND declarations are ignored
in batch mode"
  (interactive)
  (setq org-html-preamble "<i>Last Updated: %T</i>")
  (setq org-html-postamble "<hr/> <i> Author: %a (%e) <br/> Date: %T <br/> </i>")
)
       

(setq org-html-htmlize-output-type nil) ;; not sure why but this prevents htmlize from coloring example code
; (setq org-html-htmlize-output-type 'inline-css)

(add-hook 'org-mode-hook
 '(lambda ()
    ;; Move single table cells
    (local-set-key [C-M-up] (quote org-table-move-single-cell-up))
    (local-set-key [C-M-down] (quote org-table-move-single-cell-down))
    (local-set-key [C-M-left] (quote org-table-move-single-cell-left))
    (local-set-key [C-M-right] (quote org-table-move-single-cell-right))

    (local-set-key "\C-c\C-n" 'org-next-item)
    (local-set-key "\C-c\C-p" 'org-previous-item)
    (local-set-key "\C-cc" 'compile)
    (local-set-key "\C-c2" 'org-two-column)
    (local-set-key "\C-ce" 'org-example)
    (local-set-key "\C-cq" 'org-quote)
    (local-set-key "\C-ci" 'org-image)
    (local-set-key "\C-cd" 'insert-date-heading)
    (local-set-key "\C-c\C-a" 'org-agenda)
    (local-set-key "\C-\M-z" (quote format-float-column))
    ;; (local-set-key "\C-cc" 'org-table-delete-column)
    (local-set-key "\C-cm" 'org-export-latex-and-compile)
    (local-set-key "\C-cx" 'org-latex-tag)
    (local-set-key "\C-ch" 'org-html-tag)
    (local-set-key "\C-cw" 'org-html-image-attribute)
    (local-set-key "\C-c\C-d" 'insert-date)
    (local-set-key "\M-}" 'forward-paragraph)
    (local-set-key "\M-{" 'backward-paragraph)
    (local-set-key "\C-cf" 'org-latex-fontsize)

    (local-unset-key [C-tab])
    
    (auto-fill-mode 1)
    (flyspell-mode 1)
    (setq org-agenda-window-setup 'other-window)
    (setq ispell-check-comments nil)
    ;; (require 'filladapt)
    ;; (filladapt-mode)
    ;; (auto-complete-mode 1)))
    ;; (message "Finished with org-mode-hook")
    ))


;; Some code to deal with floats
(defun float-bounds-of-float-at-point ()
  "Return the start and end points of an float at the current
point.  The result is a paired list of character positions for an
float located at the current point in the current buffer."
  (save-excursion
    (let ((valid "-+0123456789.eE"))
      (skip-chars-backward valid)
      (cons (point) (+ (point) (skip-chars-forward valid))))))

(put 'float 'bounds-of-thing-at-point
     'float-bounds-of-float-at-point)
(bounds-of-thing-at-point 'float)

(defcustom float-format "%6.2f"
  "Format for floating point insertion, change at will"
  :type '(string))

(defun format-float ()
  "Format the next float or float under the point"
  (interactive)
  (skip-syntax-forward "-")		;Skip whitespace
  (when (looking-at "[0-9+-]")
    (let ((b (bounds-of-thing-at-point 'float)))
      (insert 
       (format float-format
	       (string-to-number
		(delete-and-extract-region (car b) (cdr b))))))))
	      
(defun format-float-column ()
  "Format a column of floats in an org-mode table according to
format-float"
  (interactive)
  (let ((col (org-table-current-column))
	(line (line-number-at-pos))
	(endl (line-number-at-pos (marker-position (org-table-end))))
	(pos (move-marker (make-marker) (point)))) ; Allows fedit-line-down to work
    (org-set-local 'org-pos pos)
    (dotimes (i (+ 1 (- endl (line-number-at-pos))))
      (skip-syntax-forward "-")		;Skip whitespace
      (when (looking-at "[0-9+-]")
	(let* ((field (org-table-get-field))
	       (repl (format (concat " " float-format) (string-to-number field))))
	  (org-table-get-field nil repl)))
	(org-table-fedit-line-down))
    (goto-line line)
    (org-table-goto-column col)
    (org-table-recalculate)))


(defun org-table-move-single-cell-up ()
  "Move a single cell up in a table; swap with anything in target cell"
  (interactive)
  (org-table-move-single-cell 'up))

(defun org-table-move-single-cell-down ()
  "Move a single cell down in a table; swap with anything in target cell"
  (interactive)
  (org-table-move-single-cell 'down))

(defun org-table-move-single-cell-left ()
  "Move a single cell left in a table; swap with anything in target cell"
  (interactive)
  (org-table-move-single-cell 'left))

(defun org-table-move-single-cell-right ()
  "Move a single cell right in a table; swap with anything in target cell"
  (interactive)
  (org-table-move-single-cell 'right))
	
(defun org-table-move-single-cell (direction)
  "Move the current cell in a cardinal direction according to the
  parameter symbol: 'up 'down 'left 'right. Swaps contents of
  adjacent cell with current one."
  (unless (org-at-table-p)
    (error "No table at point"))
  (let ((drow 0) (dcol 0))
    (cond ((equal direction 'up)    (setq drow -1))
	  ((equal direction 'down)  (setq drow +1))
	  ((equal direction 'left)  (setq dcol -1))
	  ((equal direction 'right) (setq dcol +1))
	  (t (error "Not a valid direction, must be one of 'up 'down 'left 'right")))
    (let* ((row1 (org-table-current-line))
	   (col1 (org-table-current-column))
	   (row2 (+ row1 drow))
	   (col2 (+ col1 dcol)))
      (unless (< col2 1)                ;Don't move off the left side of the table
        (org-table-swap-cells row1 col1 row2 col2)
        (org-table-goto-line row2)
        (org-table-goto-column col2)))))

(defun org-table-swap-cells (row1 col1 row2 col2)
  "Swap two cells indicated by the coordinates provided"
  (let ((content1 (org-table-get row1 col1))
	(content2 (org-table-get row2 col2)))
    (org-table-put row1 col1 content2)
    (org-table-put row2 col2 content1)
    (org-table-align)))

;; MARKDOWN MODE
(add-hook 'markdown-mode-hook
 '(lambda ()
    (local-set-key "\M-n" 'markdown-outline-next)
    (local-set-key "\M-p" 'markdown-outline-previous)
    ))


;; Taken from http://www.emacswiki.org/cgi-bin/wiki/ToggleWindowSplit
(defun toggle-window-split ()
  "Switch between vertical/horizontal window setup.
Vertical split shows more of each line, horizontal split shows
more lines. This code toggles between them. It only works for
frames with exactly two windows. The top window goes to the left
or vice-versa."
  (interactive)
  (if (= (count-windows) 2)
      (let* ((this-win-buffer (window-buffer))
	     (next-win-buffer (window-buffer (next-window)))
	     (this-win-edges (window-edges (selected-window)))
	     (next-win-edges (window-edges (next-window)))
	     (this-win-2nd (not (and (<= (car this-win-edges)
					 (car next-win-edges))
				     (<= (cadr this-win-edges)
					 (cadr next-win-edges)))))
	     (splitter
	      (if (= (car this-win-edges)
		     (car (window-edges (next-window))))
		  'split-window-horizontally
		'split-window-vertically)))
	(delete-other-windows)
	(let ((first-win (selected-window)))
	  (funcall splitter)
	  (if this-win-2nd (other-window 1))
	  (set-window-buffer (selected-window) this-win-buffer)
	  (set-window-buffer (next-window) next-win-buffer)
	  (select-window first-win)
	  (if this-win-2nd (other-window 1))))))




(custom-set-variables
 ;; custom-set-variables was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(TeX-PDF-mode t)
 '(TeX-view-program-list (quote (("mupdf" "mupdf %o"))))
 '(TeX-view-program-selection
   (quote
    (((output-dvi style-pstricks)
      "dvips and gv")
     (output-dvi "xdvi")
     (output-pdf "mupdf")
     (output-html "xdg-open"))))
 '(ansi-color-faces-vector
   [default bold shadow italic underline bold bold-italic bold])
 '(ansi-color-names-vector
   ["black" "red3" "green3" "yellow3" "blue2" "magenta3" "cyan3" "white"])
 '(auto-save-interval 0)
 '(auto-save-timeout 300)
 '(blink-cursor-mode nil)
 '(browse-kill-ring-quit-action (quote bury-buffer))
 '(browse-kill-ring-show-preview nil)
 '(browse-url-browser-function (quote browse-url-chromium))
 '(bs-alternative-configuration "files")
 '(bs-default-configuration "all-intern-last")
 '(c-offsets-alist (quote ((brace-list-intro . 2) (case-label . 2))))
 '(column-number-mode t)
 '(comment-fill-column 500)
 '(compilation-message-face (quote default))
 '(compile-command "make ")
 '(cua-enable-cua-keys nil)
 '(cua-global-mark-cursor-color "#2aa198")
 '(cua-normal-cursor-color "#839496")
 '(cua-overwrite-cursor-color "#b58900")
 '(cua-read-only-cursor-color "#859900")
 '(custom-safe-themes
   (quote
    ("c77439207923a1c67963c7bbe7b1867b72fe7df49cdd45439e0009621f065d8a" "13a8eaddb003fd0d561096e11e1a91b029d3c9d64554f8e897b2513dbf14b277" "2540689fd0bc5d74c4682764ff6c94057ba8061a98be5dd21116bf7bf301acfb" "d8dc153c58354d612b2576fea87fe676a3a5d43bcc71170c62ddde4a1ad9e1fb" "830877f4aab227556548dc0a28bf395d0abe0e3a0ab95455731c9ea5ab5fe4e1" "0fffa9669425ff140ff2ae8568c7719705ef33b7a927a0ba7c5e2ffcfac09b75" "7f1d414afda803f3244c6fb4c2c64bea44dac040ed3731ec9d75275b9e831fe5" "51ec7bfa54adf5fff5d466248ea6431097f5a18224788d0bd7eb1257a4f7b773" "c433c87bd4b64b8ba9890e8ed64597ea0f8eb0396f4c9a9e01bd20a04d15d358" "2809bcb77ad21312897b541134981282dc455ccd7c14d74cc333b6e549b824f3" "bfcc767f162c93ce14f7fb2fd6831225a393f02d0023d47c7c7032e59f89c234" "f7574cefc2b403e18d33c388af7785dd6b86aebd9ee29306c885a747cbdf295f" "11920f41d3143e50b6d93430a14f48d6dbfca9af378cd86ed7a52801f2af5cf6" "0da278dd1296b023b1cb4650bb1d9756db8860c6770b22e5bb03f1e7e1b78184" "85b3eda32f051e0d64cba2d43434e8fb3cf4e36db5e776758b929f5987f05068" "ffbde271521548055db522e18ebe77b137620b40a4221e4d0516100f7fd84e75" "1117f8e7f98f147218a454fce8ad7cef3b01d574d9c72b47c2db9d680d5e31b5" "9adc852a6271ec44d71fa4122bad3d4c05677d96bffb2e32a47ee39090cc887d" "5ae752c27ad89783d97a3251eb5b86ec1b81d31f4581042a6332055648093840" "c1eea9eaa6b1b78c43ffd62df435f175097318bbbc87abd29db7cac42b78a9fe" "75a5b917219cd42721057f8c9ccc5f8cbb436aefc1039d8250a494010846737e" "aa8c7f8bd6a66c2b643f24f2bd44d3ac6676454cda3ce973b4f840cab73a049d" "a2717b186d3ff0729957704a295fb04f090992ad810fc5ec44e7b49d32de0838" "07ef3ad7194bbaa56d31cb2973aa4b45f958e5a5530324f9cec95bde3464e629" "5c18638abcdb170e57a5cc8899821e4631439f2d3644ed357de3fec4de711d20" "7c7bf16e0d8b399c4816e22a03df5e989c3c4fdb6acd21871b155e1c5e3faece" "5d010fa70ee6efb060deec71793fc42d9e9dcffb5b0b144a199589319238f1d5" "3f5677846f4d7a68ebaa7ea2ebc32a32c98aa5927ee3d754771cccebc548e459" "f0a27a7317b1545a0688b2a7673dd4f719917e10d4cb00b61f1150fef743af25" "f278037813b0fc2548035ad290385c09666af76a6f9f0c80aff1dd599cbeb016" "de2b44fca2fd009c6362b9938ab458a3981f02de61b52b2a7c297166466fc332" "605bea6eb01ede54ca1cd96b3b99b23705464a3370f6752fc86513eff2f26efa" "190f3ce8cbdde46e71f3c8f7f3dfce4421781810106825bd28934bcaaea999eb" "238d6d403dc50b1ae473ce3ef520771dbf21eda6877997116ae54b6f688e9b45" "f296d6e4a85043c617d1979d8a1b40ca537fc0368a4f772c3d9f208b62c520ec" "f73c5b9483b0cd9b89df0ec7d6654d0d0b912309e361c28b841a0dc9009d427b" "6d0383c83506f04472e48db4540e75910d96b25273e487000eff2b510ae3125f" "77fb2ab10d946adc28de24f8c3de29c1825074135c0ea52c32c5a45aa91fee09" "83ad733327368c156ae59cec8f6061e598536326b1e5152622b277c9144057af" "5041706209c29a04bf7c8e52565d6716fe2070cc5617c3259553500f7e7dc2a3" "edfc1c7b56d2491dac48a598f00d12ec478cbc4b494d4077f414867948c15b20" "e1dae5fba2fe55f6d12de43b6f88bf280dedf8bea2e088e4a0e954d7afc3711d" "2d1e5d3dcd78f1bb3054efc72f648878c1e1ae4ea57b67a807660e2f74aea1e9" "1f3e368bd9c1ed296dc720c808680b241342a53602bb5cf1a58de0507144d71c" "33cfd79da8939533fd0430682a956162db7976d5b71daadba05be519d805099c" "170859a0b50754ffafa81f886a9117f6071feaae099c4c03939c85ce6b0c2988" "e11aa281b32b80441cf7a114f205a450001c059d56125f6534b6c15529fa5589" "b4eda76c7cf1a7cb4efc046004b0f0b50f30dac5d7d73a1c8a1d236336a9a29d" "8f4ec8b16c378426a21bf651b77232ffd5f450ab04c4be8e19d3d416576cd3f1" "453814065782ee81ecf56d8322194962061dc132b69e74169a276abc9a4f0bfd" "b57bd7e01b4dfeb058270d58ed6910aa052a07b292e20983783f7988f696f96c" "4fb3e470e51c9d04dd733640f698c635e5330f06badf86f1e770d7817b576f66" "c4cd7bc5ae8c05e8ae8ce90e5908f58caa465f80a92035add4372db35f611c48" "efc0693ec72fa0df91c2cf5fa87e6b518d5bc5c691d020ef6fa699e6eec91078" "9a9fdbff67305e8c5be0d94fa89f573a0f065498f0ef6f871dced244c6d29b3d" "d593d0daabcd0a508221d7d0dff750314a387bc3ae65bc948732d5a2f8a18dc1" "1b5aeec9f203997ac38f8e22408377602499ba983359eace44fdc38482b2fe43" "f1ce4d9faba2b9d52e151a91b951a522c049dd57c2d4102ba1b97d4066810191" "51e41bc78f65dc67a6f0b3ec09a62fc0b3c1520c4343555ea21e9139a8ef0e67" "ffc00c0353835098a7553dd061c4bbee956d4bf3fc8f5fa4eca2de02acad6b3e" "97fe2c84c7c11acb13ccef96666368b4c8123d41610649c034fa8962ee96ee69" "02db4409ddb1e04976cddd6ee6fb7bafdbdb8514b801ea1e342f4e4923de912d" "3748be2a2a6a16ec1c7e3d2881ee6b95ecc10983baa469c1cdcf24e251833f80" "84b1cc6936e09a2b4aefa5ada43caf9054fd8a72dd81350e2cabbe2c4ba46c6f" "6eaead8aea8c457e06ef606e02af9a2534e8196dca376fc5d0f3edf5be8d409c" "f388f3addf5da71405fd70f86ffe6534a587ae20935f4cf1bac7366f8cd1b8d9" "52147aad05a4217e744a90fa666625fd23f660db741127173514ab1a45c1f75f" "dab83819b283cda38a66a15531cd239f1ba90a1aebc8f777d47415a2222fbc3b" "38bb38113f28ed001299aba6a309bb4207b8d10676ad3877473fb0beca13c662" "e2c8380494916911e330be21e2f8aac41567fd6b78867c5537b27f3bb0d08f1e" "e89db0ca368b8f8b6c268f589bc3c2d3f78981ffd73d434e6ac0448b925c9a2e" "892a115f510521893c502094210b7a5d11157ce981e6426e1d01ad19daf01c93" default)))
 '(dired-dwim-target nil)
 '(dired-listing-switches "-ahog")
 '(dired-recursive-copies (quote always))
 '(ediff-diff-options "-w")
 '(ediff-split-window-function (quote split-window-horizontally))
 '(ediff-window-setup-function (quote ediff-setup-windows-plain))
 '(fci-rule-color "#073642")
 '(float-format "%6.3f")
 '(global-hl-line-mode t)
 '(gnuplot-buffer-max-size 10000)
 '(graphviz-dot-auto-indent-on-semi nil)
 '(graphviz-dot-indent-width 2)
 '(grep-highlight-matches nil)
 '(helm-always-two-windows nil)
 '(helm-full-frame nil)
 '(helm-split-window-in-side-p t)
 '(highlight-changes-colors (quote ("#d33682" "#6c71c4")))
 '(highlight-symbol-colors
   (quote
    ("#3b2b40b432a1" "#07ab45f64ce9" "#475733ea3554" "#1d623c04567f" "#2d5343d8332c" "#436f35f73166" "#0613413e597e")))
 '(highlight-symbol-foreground-color "#93a1a1")
 '(highlight-tail-colors
   (quote
    (("#073642" . 0)
     ("#5b7300" . 20)
     ("#007d76" . 30)
     ("#0061a8" . 50)
     ("#866300" . 60)
     ("#992700" . 70)
     ("#a00559" . 85)
     ("#073642" . 100))))
 '(hl-bg-colors
   (quote
    ("#866300" "#992700" "#a7020a" "#a00559" "#243e9b" "#0061a8" "#007d76" "#5b7300")))
 '(hl-fg-colors
   (quote
    ("#002b36" "#002b36" "#002b36" "#002b36" "#002b36" "#002b36" "#002b36" "#002b36")))
 '(hl-paren-colors (quote ("#2aa198" "#b58900" "#268bd2" "#6c71c4" "#859900")))
 '(ido-enable-tramp-completion nil)
 '(ido-file-extensions-order (quote (".org" ".java" ".c" ".txt" "/" ".tex" ".zip")))
 '(ido-ignore-files
   (quote
    ("\\`CVS/" "\\`#" "\\`.#" "\\`\\.\\./" "\\`\\./" "\\.\\(annot\\|cm[iox]\\)$")))
 '(ido-max-dir-file-cache 5)
 '(ido-max-file-prompt-width 0.35)
 '(ido-max-prospects 8)
 '(ido-max-work-directory-list 2)
 '(indent-tabs-mode nil)
 '(inferior-lisp-program "~/bin/clj ")
 '(inhibit-startup-screen t)
 '(lisp-indent-maximum-backtracking 1)
 '(lisp-indent-offset nil)
 '(lisp-loop-keyword-indentation 1)
 '(lisp-simple-loop-indentation 1)
 '(lisp-tag-body-indentation 1)
 '(lsp-ui-doc-border "#93a1a1")
 '(matlab-auto-fill nil)
 '(matlab-cont-level 2)
 '(matlab-indent-function-body nil)
 '(matlab-indent-level 2)
 '(matlab-shell-ask-MATLAB-for-completions t)
 '(matlab-shell-command-switches (quote ("-nodesktop" "-nosplash")))
 '(matlab-shell-input-ring-size 500)
 '(nrepl-message-colors
   (quote
    ("#dc322f" "#cb4b16" "#b58900" "#5b7300" "#b3c34d" "#0061a8" "#2aa198" "#d33682" "#6c71c4")))
 '(org-agenda-files
   (quote
    ("~/testy/bc-echo.org" "~/2041-F2018/assignments/a5-calculon/a5.org" "~/2011-R2018/lectures/07-induction.org" "~/2011-R2018/lectures/04-sets.org")))
 '(org-confirm-babel-evaluate nil)
 '(org-ditaa-jar-path "/usr/share/java/ditaa/ditaa-0_9.jar")
 '(org-export-allow-bind-keywords t)
 '(org-export-in-background nil)
 '(org-list-allow-alphabetical t)
 '(org-src-fontify-natively t)
 '(package-selected-packages
   (quote
    (abyss-theme afternoon-theme ahungry-theme solarized-theme ess-smart-underscore racket-mode nyan-mode powerline arduino-mode scad-mode htmlize browse-kill-ring god-mode 2048-game evil mmm-mode magit cider php-scratch lua-mode javadoc-lookup expand-region)))
 '(pos-tip-background-color "#073642")
 '(pos-tip-foreground-color "#93a1a1")
 '(proced-format-alist
   (quote
    ((short user pid tree pcpu pmem rss start time
            (args comm))
     (medium user pid tree pcpu pmem vsize rss ttname state start time
             (args comm))
     (long user euid group pid tree pri nice pcpu pmem vsize rss ttname state start time
           (args comm))
     (verbose user euid group egid pid ppid tree pgrp sess pri nice pcpu pmem state thcount vsize rss ttname tpgid minflt majflt cminflt cmajflt start time utime stime ctime cutime cstime etime
              (args comm)))))
 '(proced-grammar-alist
   (quote
    ((euid "EUID" "%d" right proced-< nil
           (euid pid)
           (nil t nil))
     (user "USER" nil left proced-string-lessp nil
           (user pid)
           (nil t nil))
     (egid "EGID" "%d" right proced-< nil
           (egid euid pid)
           (nil t nil))
     (group "GROUP" nil left proced-string-lessp nil
            (group user pid)
            (nil t nil))
     (comm "COMMAND" nil left proced-string-lessp nil
           (comm pid)
           (nil t nil))
     (state "STAT" nil left proced-string-lessp nil
            (state pid)
            (nil t nil))
     (ppid "PPID" "%d" right proced-< nil
           (ppid pid)
           ((lambda
              (ppid)
              (proced-filter-parents proced-process-alist ppid))
            "refine to process parents"))
     (pgrp "PGRP" "%d" right proced-< nil
           (pgrp euid pid)
           (nil t nil))
     (sess "SESS" "%d" right proced-< nil
           (sess pid)
           (nil t nil))
     (ttname "TTY" proced-format-ttname left proced-string-lessp nil
             (ttname pid)
             (nil t nil))
     (tpgid "TPGID" "%d" right proced-< nil
            (tpgid pid)
            (nil t nil))
     (minflt "MINFLT" "%d" right proced-< nil
             (minflt pid)
             (nil t t))
     (majflt "MAJFLT" "%d" right proced-< nil
             (majflt pid)
             (nil t t))
     (cminflt "CMINFLT" "%d" right proced-< nil
              (cminflt pid)
              (nil t t))
     (cmajflt "CMAJFLT" "%d" right proced-< nil
              (cmajflt pid)
              (nil t t))
     (utime "UTIME" proced-format-time right proced-time-lessp t
            (utime pid)
            (nil t t))
     (stime "STIME" proced-format-time right proced-time-lessp t
            (stime pid)
            (nil t t))
     (time "TIME" proced-format-time right proced-time-lessp t
           (time pid)
           (nil t t))
     (cutime "CUTIME" proced-format-time right proced-time-lessp t
             (cutime pid)
             (nil t t))
     (cstime "CSTIME" proced-format-time right proced-time-lessp t
             (cstime pid)
             (nil t t))
     (ctime "CTIME" proced-format-time right proced-time-lessp t
            (ctime pid)
            (nil t t))
     (pri "PR" "%d" right proced-< t
          (pri pid)
          (nil t t))
     (nice "NI" "%3d" 3 proced-< t
           (nice pid)
           (t t nil))
     (thcount "THCOUNT" "%d" right proced-< t
              (thcount pid)
              (nil t t))
     (start "START" proced-format-start 6 proced-time-lessp nil
            (start pid)
            (t t nil))
     (vsize "VSIZE" "%d" right proced-< t
            (vsize pid)
            (nil t t))
     (rss "RSS" print-mem right proced-< t
          (rss pid)
          (nil t t))
     (etime "ETIME" proced-format-time right proced-time-lessp t
            (etime pid)
            (nil t t))
     (pcpu "%CPU" "%.1f" right proced-< t
           (pcpu pid)
           (nil t t))
     (pmem "%MEM" "%.1f" right proced-< t
           (pmem pid)
           (nil t t))
     (args "ARGS" proced-format-args left proced-string-lessp nil
           (args pid)
           (nil t nil))
     (pid "PID" "%d" right proced-< nil
          (pid)
          ((lambda
             (ppid)
             (proced-filter-children proced-process-alist ppid))
           "refine to process children"))
     (tree "TREE" proced-format-tree left nil nil nil nil))))
 '(processing-application-dir nil)
 '(processing-location "/usr/bin/processing-java")
 '(processing-sketchbook-dir "\"~/Dropbox/aids/processing\"")
 '(python-indent-offset 2)
 '(python-shell-interpreter "python3")
 '(quack-programs
   (quote
    ("mzscheme" "bigloo" "csi" "csi -hygienic" "gosh" "gracket" "gsi" "gsi ~~/syntax-case.scm -" "guile" "kawa" "mit-scheme" "racket" "racket -il typed/racket" "rs" "scheme" "scheme48" "scsh" "sisc" "stklos" "sxi")))
 '(scheme-program-name "/soft/plt-4.2.5/ubuntuamd1/bin/mzscheme")
 '(select-enable-clipboard t)
 '(show-paren-mode t)
 '(smartrep-mode-line-active-bg (solarized-color-blend "#859900" "#073642" 0.2))
 '(term-default-bg-color "#002b36")
 '(term-default-fg-color "#839496")
 '(tool-bar-mode nil)
 '(transient-mark-mode nil)
 '(tuareg-in-indent 0)
 '(tuareg-match-indent 2)
 '(tuareg-with-indent 0)
 '(uniquify-buffer-name-style (quote forward) nil (uniquify))
 '(vc-annotate-background nil)
 '(vc-annotate-background-mode nil)
 '(vc-annotate-color-map
   (quote
    ((20 . "#dc322f")
     (40 . "#ca7966832090")
     (60 . "#c05578c91534")
     (80 . "#b58900")
     (100 . "#a6088eed0000")
     (120 . "#9e3a91a60000")
     (140 . "#9628943b0000")
     (160 . "#8dc596ad0000")
     (180 . "#859900")
     (200 . "#76ef9b6045e8")
     (220 . "#6cd69ca95b9d")
     (240 . "#5f5f9e06701f")
     (260 . "#4c1a9f778424")
     (280 . "#2aa198")
     (300 . "#3002984eaf4d")
     (320 . "#2f6f93e8bae0")
     (340 . "#2c598f79c66f")
     (360 . "#268bd2"))))
 '(vc-annotate-very-old-color nil)
 '(vc-delete-logbuf-window nil)
 '(vc-follow-symlinks t)
 '(vm-infer-mime-types t)
 '(vm-mime-attachment-infer-type-for-text-attachments t)
 '(vm-mime-delete-viewer-processes nil)
 '(vm-preview-lines nil)
 '(weechat-color-list
   (quote
    (unspecified "#002b36" "#073642" "#a7020a" "#dc322f" "#5b7300" "#859900" "#866300" "#b58900" "#0061a8" "#268bd2" "#a00559" "#d33682" "#007d76" "#2aa198" "#839496" "#657b83")))
 '(xterm-color-names
   ["#073642" "#dc322f" "#859900" "#b58900" "#268bd2" "#d33682" "#2aa198" "#eee8d5"])
 '(xterm-color-names-bright
   ["#002b36" "#cb4b16" "#586e75" "#657b83" "#839496" "#6c71c4" "#93a1a1" "#fdf6e3"]))


;; CRUFT
;; No more!

(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 )
